@@ -1,6 +1,6 @@
-/* 
+/*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2010 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2013 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -28,36 +28,39 @@
 #include <sys/types.h>
 
 #include "config.h"
-#include "dhcpcd.h"
+#include "dhcp.h"
 #include "if-pref.h"
 #include "net.h"
 
 /* Interface comparer for working out ordering. */
 static int
-ifcmp(struct interface *si, struct interface *ti)
+ifcmp(const struct interface *si, const struct interface *ti)
 {
 	int sill, till;
+	const struct dhcp_state *sis, *tis;
 
-	if (si->state && !ti->state)
+	sis = D_CSTATE(si);
+	tis = D_CSTATE(ti);
+	if (sis && !tis)
 		return -1;
-	if (!si->state && ti->state)
+	if (!sis && tis)
 		return 1;
-	if (!si->state && !ti->state)
+	if (!sis && !tis)
 		return 0;
 	/* If one has a lease and the other not, it takes precedence. */
-	if (si->state->new && !ti->state->new)
+	if (sis->new && !tis->new)
 		return -1;
-	if (!si->state->new && ti->state->new)
+	if (!sis->new && tis->new)
 		return 1;
 	/* If we are either, they neither have a lease, or they both have.
 	 * We need to check for IPv4LL and make it non-preferred. */
-	if (si->state->new && ti->state->new) {
-		sill = (si->state->new->cookie == htonl(MAGIC_COOKIE));
-		till = (ti->state->new->cookie == htonl(MAGIC_COOKIE));
+	if (sis->new && tis->new) {
+		sill = (sis->new->cookie == htonl(MAGIC_COOKIE));
+		till = (tis->new->cookie == htonl(MAGIC_COOKIE));
 		if (!sill && till)
-			return -1;
-		if (sill && !till)
 			return 1;
+		if (sill && !till)
+			return -1;
 	}
 	/* Then carrier status. */
 	if (si->carrier > ti->carrier)
@@ -76,33 +79,27 @@ ifcmp(struct interface *si, struct interface *ti)
 void
 sort_interfaces(void)
 {
-	struct interface *sorted, *ifp, *ifn, *ift;
+	struct if_head sorted;
+	struct interface *ifp, *ift;
 
-	if (!ifaces || !ifaces->next)
+	if (ifaces == NULL ||
+	    (ifp = TAILQ_FIRST(ifaces)) == NULL ||
+	    TAILQ_NEXT(ifp, next) == NULL)
 		return;
-	sorted = ifaces;
-	ifaces = ifaces->next;
-	sorted->next = NULL;
-	for (ifp = ifaces; ifp && (ifn = ifp->next, 1); ifp = ifn) {
-		/* Are we the new head? */
-		if (ifcmp(ifp, sorted) == -1) {
-			ifp->next = sorted;
-			sorted = ifp;
-			continue;
-		}
-		/* Do we fit in the middle? */
-		for (ift = sorted; ift->next; ift = ift->next) {
-			if (ifcmp(ifp, ift->next) == -1) {
-				ifp->next = ift->next;
-				ift->next = ifp;
+
+	TAILQ_INIT(&sorted);
+	TAILQ_REMOVE(ifaces, ifp, next);
+	TAILQ_INSERT_HEAD(&sorted, ifp, next);
+	while ((ifp = TAILQ_FIRST(ifaces))) {
+		TAILQ_REMOVE(ifaces, ifp, next);
+		TAILQ_FOREACH(ift, &sorted, next) {
+			if (ifcmp(ifp, ift) == -1) {
+				TAILQ_INSERT_BEFORE(ift, ifp, next);
 				break;
 			}
 		}
-		/* We must be at the end */
-		if (!ift->next) {
-			ift->next = ifp;
-			ifp->next = NULL;
-		}
+		if (ift == NULL)
+			TAILQ_INSERT_TAIL(&sorted, ifp, next);
 	}
-	ifaces = sorted;
+	TAILQ_CONCAT(ifaces, &sorted, next);
 }
