@@ -1,6 +1,6 @@
-/* 
+/*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2011 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2013 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -34,20 +34,22 @@
 
 #include "arp.h"
 #include "common.h"
-#include "dhcpcd.h"
+#include "dhcp.h"
 #include "eloop.h"
 #include "if-options.h"
 #include "ipv4ll.h"
 #include "net.h"
 
 static struct dhcp_message *
-make_ipv4ll_lease(uint32_t addr)
+ipv4ll_make_lease(uint32_t addr)
 {
 	uint32_t u32;
 	struct dhcp_message *dhcp;
 	uint8_t *p;
 
-	dhcp = xzalloc(sizeof(*dhcp));
+	dhcp = calloc(1, sizeof(*dhcp));
+	if (dhcp == NULL)
+		return NULL;
 	/* Put some LL options in */
 	dhcp->yiaddr = addr;
 	p = dhcp->options;
@@ -67,7 +69,7 @@ make_ipv4ll_lease(uint32_t addr)
 }
 
 static struct dhcp_message *
-find_ipv4ll_lease(uint32_t old_addr)
+ipv4ll_find_lease(uint32_t old_addr)
 {
 	uint32_t addr;
 
@@ -79,78 +81,84 @@ find_ipv4ll_lease(uint32_t old_addr)
 		    IN_LINKLOCAL(ntohl(addr)))
 			break;
 	}
-	return make_ipv4ll_lease(addr);
+	return ipv4ll_make_lease(addr);
 }
 
 void
-start_ipv4ll(void *arg)
+ipv4ll_start(void *arg)
 {
-	struct interface *iface = arg;
+	struct interface *ifp = arg;
+	struct dhcp_state *state = D_STATE(ifp);
 	uint32_t addr;
 
-	delete_timeout(NULL, iface);
-	iface->state->probes = 0;
-	iface->state->claims = 0;
-	if (iface->addr.s_addr) {
-		iface->state->conflicts = 0;
-		if (IN_LINKLOCAL(htonl(iface->addr.s_addr))) {
-			send_arp_announce(iface);
+	eloop_timeout_delete(NULL, ifp);
+	state->probes = 0;
+	state->claims = 0;
+	if (state->addr.s_addr) {
+		state->conflicts = 0;
+		if (IN_LINKLOCAL(htonl(state->addr.s_addr))) {
+			arp_announce(ifp);
 			return;
 		}
 	}
 
-	if (iface->state->offer == NULL)
+	if (state->offer == NULL)
 		addr = 0;
 	else {
-		addr = iface->state->offer->yiaddr;
-		free(iface->state->offer);
+		addr = state->offer->yiaddr;
+		free(state->offer);
 	}
 	/* We maybe rebooting an IPv4LL address. */
 	if (!IN_LINKLOCAL(htonl(addr))) {
 		syslog(LOG_INFO, "%s: probing for an IPv4LL address",
-		    iface->name);
+		    ifp->name);
 		addr = 0;
 	}
 	if (addr == 0)
-		iface->state->offer = find_ipv4ll_lease(addr);
+		state->offer = ipv4ll_find_lease(addr);
 	else
-		iface->state->offer = make_ipv4ll_lease(addr);
-	iface->state->lease.frominfo = 0;
-	send_arp_probe(iface);
+		state->offer = ipv4ll_make_lease(addr);
+	if (state->offer == NULL)
+		syslog(LOG_ERR, "%s: %m", __func__);
+	else {
+		state->lease.frominfo = 0;
+		arp_probe(ifp);
+	}
 }
 
 void
-handle_ipv4ll_failure(void *arg)
+ipv4ll_handle_failure(void *arg)
 {
-	struct interface *iface = arg;
+	struct interface *ifp = arg;
+	struct dhcp_state *state = D_STATE(ifp);
 	time_t up;
 
-	if (iface->state->fail.s_addr == iface->addr.s_addr) {
+	if (state->fail.s_addr == state->addr.s_addr) {
 		up = uptime();
-		if (iface->state->defend + DEFEND_INTERVAL > up) {
+		if (state->defend + DEFEND_INTERVAL > up) {
 			syslog(LOG_DEBUG,
 			    "%s: IPv4LL %d second defence failed",
-			    iface->name, DEFEND_INTERVAL);
-			drop_dhcp(iface, "EXPIRE");
-			iface->state->conflicts = -1;
+			    ifp->name, DEFEND_INTERVAL);
+			dhcp_drop(ifp, "EXPIRE");
+			state->conflicts = -1;
 		} else {
 			syslog(LOG_DEBUG, "%s: defended IPv4LL address",
-			    iface->name);
-			iface->state->defend = up;
+			    ifp->name);
+			state->defend = up;
 			return;
 		}
 	}
 
-	close_sockets(iface);
-	free(iface->state->offer);
-	iface->state->offer = NULL;
-	delete_timeout(NULL, iface);
-	if (++iface->state->conflicts > MAX_CONFLICTS) {
+	dhcp_close(ifp);
+	free(state->offer);
+	state->offer = NULL;
+	eloop_timeout_delete(NULL, ifp);
+	if (++state->conflicts > MAX_CONFLICTS) {
 		syslog(LOG_ERR, "%s: failed to acquire an IPv4LL address",
-		    iface->name);
-		iface->state->interval = RATE_LIMIT_INTERVAL / 2;
-		start_discover(iface);
+		    ifp->name);
+		state->interval = RATE_LIMIT_INTERVAL / 2;
+		dhcp_discover(ifp);
 	} else {
-		add_timeout_sec(PROBE_WAIT, start_ipv4ll, iface);
+		eloop_timeout_add_sec(PROBE_WAIT, ipv4ll_start, ifp);
 	}
 }
