@@ -1,6 +1,6 @@
-/* 
+/*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2012 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2013 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -32,10 +32,11 @@
 #include <syslog.h>
 
 #include "common.h"
+#include "dhcpcd.h"
 #include "if-options.h"
 #include "platform.h"
 
-static const char *mproc = 
+static const char *mproc =
 #if defined(__alpha__)
 	"system type"
 #elif defined(__arm__)
@@ -73,8 +74,10 @@ static const char *mproc =
 #endif
 	;
 
-char **restore;
-ssize_t nrestore;
+#ifdef INET6
+static char **restore;
+static ssize_t nrestore;
+#endif
 
 char *
 hardware_platform(void)
@@ -109,6 +112,7 @@ hardware_platform(void)
 	return p;
 }
 
+#ifdef INET6
 static int
 check_proc_int(const char *path)
 {
@@ -159,7 +163,7 @@ restore_kernel_ra(void)
 		       restore[nrestore]);
 		snprintf(path, sizeof(path), "%s/%s/accept_ra",
 			 prefix, restore[nrestore]);
-		if (write_path(path, "1") == -1)
+		if (write_path(path, "1") == -1 && errno != ENOENT)
 			syslog(LOG_ERR, "write_path: %s: %m", path);
 #ifdef DEBUG_MEMORY
 		}
@@ -172,52 +176,88 @@ restore_kernel_ra(void)
 }
 
 int
-check_ipv6(const char *ifname)
+check_ipv6(const char *ifname, int own)
 {
-	int r, ex, i;
-	char path[256];
+	static int ipv6_checked = 0;
+	int ra, forward, ex, i;
+	char path[256], *p, **nrest;
 
 	if (ifname == NULL) {
+		if (ipv6_checked)
+			return 1;
+		ipv6_checked = 1;
 		ifname = "all";
 		ex = 1;
 	} else
 		ex = 0;
 
 	snprintf(path, sizeof(path), "%s/%s/accept_ra", prefix, ifname);
-	r = check_proc_int(path);
-	if (r == 0)
-		options |= DHCPCD_IPV6RA_OWN;
-	else if (options & DHCPCD_IPV6RA_OWN) {
+	ra = check_proc_int(path);
+	if (ra == -1)
+		/* The sysctl probably doesn't exist, but this isn't an
+		 * error as such so just log it and continue */
+		syslog(errno == ENOENT ? LOG_DEBUG : LOG_WARNING,
+		    "%s: %m", path);
+	else if (ra != 0 && own) {
 		syslog(LOG_INFO, "%s: disabling Kernel IPv6 RA support",
 		    ifname);
 		if (write_path(path, "0") == -1) {
 			syslog(LOG_ERR, "write_path: %s: %m", path);
-			return 0;
+			return ra;
 		}
 		for (i = 0; i < nrestore; i++)
 			if (strcmp(restore[i], ifname) == 0)
 				break;
 		if (i == nrestore) {
-			restore = realloc(restore,
-			    (nrestore + 1) * sizeof(char *));
-			if (restore == NULL) {
-				syslog(LOG_ERR, "realloc: %m");
-				exit(EXIT_FAILURE);
+			p = strdup(ifname);
+			if (p == NULL) {
+				syslog(LOG_ERR, "%s: %m", __func__);
+				goto forward;
 			}
-			restore[nrestore++] = xstrdup(ifname);
+			nrest = realloc(restore,
+			    (nrestore + 1) * sizeof(char *));
+			if (nrest == NULL) {
+				syslog(LOG_ERR, "%s: %m", __func__);
+				goto forward;
+			}
+			restore = nrest;
+			restore[nrestore++] = p;
+
 		}
 		if (ex)
 			atexit(restore_kernel_ra);
 	}
 
-	if (r != 2) {
+forward:
+	if (ra != 2) {
 		snprintf(path, sizeof(path), "%s/%s/forwarding",
 		    prefix, ifname);
-		if (check_proc_int(path) != 0) {
+		forward = check_proc_int(path);
+		if (forward == -1) {
+			/* The sysctl probably doesn't exist, but this isn't an
+			 * error as such so just log it and continue */
+			syslog(errno == ENOENT ? LOG_DEBUG : LOG_WARNING,
+			    "%s: %m", path);
+		} else if (forward != 0) {
 			syslog(LOG_WARNING,
 			    "%s: configured as a router, not a host", ifname);
 			return 0;
 		}
 	}
-	return 1;
+	return ra;
 }
+
+int
+ipv6_dadtransmits(const char *ifname)
+{
+	char path[256];
+	int r;
+
+	if (ifname == NULL)
+		ifname = "default";
+
+	snprintf(path, sizeof(path), "%s/%s/dad_transmits", prefix, ifname);
+	r = check_proc_int(path);
+	return r < 0 ? 0 : r;
+}
+#endif
